package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import javax.xml.datatype.DatatypeConfigurationException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import info.textgrid.middleware.oaipmh.GetRecordType;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.RecordType;

/**
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2021-09-10
 * @since 2020-06-13
 */
public class RecordListDelivererDATACITE extends RecordListDelivererAbstract {

  private static Log log = LogFactory.getLog(RecordListDelivererDATACITE.class);

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererDATACITE(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#getRecords(java.lang.String,
   * java.lang.String, java.lang.String, java.lang.String)
   */
  public ListRecordsType getRecords(String from, String to, String set, String resumptionToken)
      throws ParseException, IOException {

    ListRecordsType openAireRecordList = new ListRecordsType();

    // Create one RecordDeliverer only, to configure all fields only ONCE.
    // FIYME May be solved using spring configuration, but I do not know how!
    RecordDelivererDATACITE openAireRecord =
        new RecordDelivererDATACITE(this.textgrid, this.dariah);
    // Must be done here, as spring does not configure this bean if instantiated with "new".
    openAireRecord.setConfigFileProps();

    List<String> URIList = getUriList(from, to, set, resumptionToken);
    for (String uri : URIList) {

      String eventuallyChangedUri = uri;

      // **
      // TextGrid
      // **

      if (this.textgrid) {
        // We must remove the prefix, as ElasticSearch is storing the IDs without it.
        eventuallyChangedUri = uri.replace("textgrid:", "");
      }

      // **
      // DARIAH
      // **

      // else if (this.dariah) {
      // Nothing to remove for DARIAH IDs here.
      // }

      log.debug("uri: " + uri + " | eventuallyChangedUri: " + eventuallyChangedUri);

      try {

        GetRecordType grt = openAireRecord.getRecordById(eventuallyChangedUri);
        RecordType record = grt.getRecord();
        // Header is already set by getRecordById().

        openAireRecordList.getRecord().add(record);

      } catch (DatatypeConfigurationException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    openAireRecordList.setResumptionToken(getResTokenForResponse());

    return openAireRecordList;
  }

  /**
   * @param from
   * @param to
   * @param set
   * @param resumptionToken
   * @return
   */
  private List<String> getUriList(String from, String to, String set, String resumptionToken) {

    List<String> result = new ArrayList<String>();

    QueryBuilder query;

    log.debug("rangefield: " + this.dateOfObjectCreation);
    log.debug("set: " + set);

    QueryBuilder rangeQuery = QueryBuilders.rangeQuery(this.dateOfObjectCreation).from(from).to(to);
    QueryBuilder tgFilterSandBox = QueryBuilders.matchPhraseQuery("nearlyPublished", "true");

    if (set != null && !set.equals("openaire_data")) {

      String queryField = "";
      String valueField = "";

      if (this.dariah == true) {
        // FIXME Put query field into config!
        queryField = "administrativeMetadata.dcterms:relation";
        valueField = set;

        // Add new record with set metadata in it (to also find collection metadata in Repository
        // Search!)

      }

      if (this.textgrid == true) {
        String[] setParts = set.split(":");
        if (setParts[0].equals("project")) {
          // FIXME Put query field into config!
          queryField = "project.id";
          valueField = setParts[1];
        }
      }

      QueryBuilder matchQuery = QueryBuilders.matchPhraseQuery(queryField, valueField);
      if (this.textgrid) {
        query =
            QueryBuilders.boolQuery().must(rangeQuery).must(matchQuery).mustNot(tgFilterSandBox);
      } else {
        query = QueryBuilders.boolQuery().must(rangeQuery).must(matchQuery);
      }
    } else {
      if (this.textgrid) {
        query = QueryBuilders.boolQuery().must(rangeQuery).mustNot(tgFilterSandBox);
      } else {
        query = QueryBuilders.boolQuery().must(rangeQuery);
      }
    }

    log.debug("query:\n" + query);

    result = getFieldsFromESIndex(query, resumptionToken, set);

    log.debug("URI list result: " + result);

    return result;
  }

}
