package info.textgrid.middleware;

import java.io.IOException;
import java.text.ParseException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchScrollRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.common.unit.TimeValue;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.json.JSONObject;
import info.textgrid.middleware.oaipmh.ListRecordsType;
import info.textgrid.middleware.oaipmh.ResumptionTokenType;

/**
 *
 */
public class RecordListDelivererIDIOM extends RecordListDelivererAbstract {

  private static Log log = LogFactory.getLog(RecordListDelivererIDIOM.class);

  /**
   * @param textgrid
   * @param dariah
   */
  public RecordListDelivererIDIOM(boolean textgrid, boolean dariah) {
    super(textgrid, dariah);
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.RecordListDelivererInterface#getRecords(java.lang.String,
   * java.lang.String, java.lang.String, java.lang.String)
   */
  public ListRecordsType getRecords(final String from, final String to, final String set,
      final String resumptionToken) throws ParseException, IOException {

    ListRecordsType recordList = new ListRecordsType();

    BoolQueryBuilder recordFilterForClassicMayan;
    RangeQueryBuilder rangeQuery;

    rangeQuery = QueryBuilders.rangeQuery("lastModified").from(from).to(to);

    recordFilterForClassicMayan = QueryBuilders.boolQuery().must(rangeQuery)
        .must(QueryBuilders.matchPhraseQuery("project.id",
            "TGPR-0e926f53-1aba-d415-ecf6-539edcd8a318"))
        .must(QueryBuilders.matchPhraseQuery("format", "text/tg.inputform+rdf+xml"))
        .must(QueryBuilders.matchPhraseQuery("notes", "ARTEFACT"));

    SearchResponse scrollResp;

    SearchRequest searchRequest = new SearchRequest("textgrid-nonpublic");
    SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

    scrollResp = null;

    searchSourceBuilder.query(recordFilterForClassicMayan);
    searchSourceBuilder.size(30);
    searchRequest.source(searchSourceBuilder);

    if (resumptionToken != null) {
      SearchScrollRequest scrollRequest = new SearchScrollRequest(resumptionToken);
      scrollRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = OAI_ESClient.getEsClient().scroll(scrollRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    } else {
      searchRequest.source(searchSourceBuilder);
      searchRequest.scroll(TimeValue.timeValueHours(24L));
      try {
        scrollResp = OAI_ESClient.getEsClient().search(searchRequest, RequestOptions.DEFAULT);
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    String scrollID = scrollResp.getScrollId();

    long completeListSize = scrollResp.getHits().totalHits;
    // long completeListSize = 45;

    if (completeListSize > 0) {
      setFoundItems(true);
      int i = 0;

      for (SearchHit hit : scrollResp.getHits().getHits()) {
        i++;

        String textgridURI =
            OAIPMHUtilities.fieldLoader(new JSONObject(hit.getSourceAsMap()), "textgridUri");

        log.debug("IDIOM URIs: " + textgridURI);

        // hit.getFields().get("textgridUri").getValue().toString().replace(".0", "");

        log.debug("Processing: " + textgridURI);

        RecordDelivererIDIOM idiomRecord = new RecordDelivererIDIOM(true, false);
        recordList.getRecord()
            .add(idiomRecord.getRecordById(textgridURI.replace(".0", "")).getRecord());
      }
      // Check the need for a resumption token!
      ResumptionTokenType resTokenForResponse = OAIPMHUtilities.getResumptionToken(
          completeListSize, resumptionToken, cursorCollector, scrollID, 30, i);
      if (resTokenForResponse != null) {
        recordList.setResumptionToken(resTokenForResponse);
      }
    } else {
      setFoundItems(false);
    }

    return recordList;
  }

}
