package info.textgrid.middleware;

import static org.junit.Assert.assertTrue;
import java.io.IOException;
import java.net.URL;
import org.apache.cxf.helpers.IOUtils;
import org.apache.http.HttpStatus;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.utils.httpclient.TGHttpClient;
import info.textgrid.utils.httpclient.TGHttpResponse;

/**
 * <p>
 * Some online tests for the DARIAH-DE Repository OAIMPH service.
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 */
@Ignore
public class OaiPmhDariahdeOnlineTests {

  // TODO Configure config files for all the different OAI-PMH service instances!

  // The OAIPMH host to be tested.
  // private static String host = "https://repository.de.dariah.eu/1.0/oaipmh/";
  // private static String host = "https://dhrepworkshop.de.dariah.eu/1.0/oaipmh/";
  private static String host = "http://trep.de.dariah.eu/1.0/oaipmh/";
  // private static String host = "http://vm1.dariah.local/1.0/oaipmh/";

  // Some output finals.
  private static final String ERROR = ">>> ERROR";
  private static final String OK = ">>> OKIDO";
  private static final String TESTING = "\n>>> TESTING ";

  // Time to wait between the single queries using resumption tokens in milliseconds.
  private static final Long TIME = 1000l;

  // **
  // PREPARATIONS
  // **

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {}

  /**
   * @throws Exception
   */
  @Before
  public void setUp() throws Exception {}

  /**
   * @throws Exception
   */
  @After
  public void tearDown() throws Exception {}

  // **
  // TESTS
  // **

  /**
   * @throws IOException
   */
  @Test
  public void testGetVersion() throws IOException {

    String shouldStartWith = "oaipmh-core";

    System.out.println(TESTING + "#GETVERSION for '" + shouldStartWith + "'");

    TGHttpResponse httpResponse = getHttpResponse(host + "oai/version");
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.startsWith(shouldStartWith)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should start with '" + shouldStartWith + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testRootWithSlash() throws IOException {

    String url = "oai/";
    String shouldContain = "DARIAH-DE Repository";

    testRootURL(url, shouldContain);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testRootWithoutSlash() throws IOException {

    String url = "oai";
    String shouldContain = "DARIAH-DE Repository";

    testRootURL(url, shouldContain);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testBlubbVerb() throws IOException {

    String url = "oai?verb=BLUBB";
    String shouldContain = "Illegal OAI verb";

    testRootURL(url, shouldContain);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testIdentify() throws IOException {

    String verb = "Identify";
    String shouldContain = "DARIAH-DE Repository";

    System.out.println(TESTING + "#IDENTIFY");

    TGHttpResponse httpResponse = getHttpResponse(host + "oai?verb=" + verb);
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.contains(shouldContain)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should contain '" + shouldContain + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListSets() throws IOException {

    // repository.de.dariah.eu
    String set = "hdl:21.11113/0000-000B-CADD-9";

    if (host.contains("vm1")) {
      // VM1: <setSpec>hdl:21.T11991/0000-001B-4CE2-F</setSpec>
      // VM1: <setName>DUNGEON MASTER images</setName>
      set = "hdl:21.T11991/0000-001B-4CE2-F";
    } else if (host.contains("trep")) {
      set = "hdl:21.T11991/0000-0008-C3AC-A";
    } else if (host.contains("dhrepworkshop")) {
      set = "hdl:21.T11991/0000-0013-8FEE-F";
    }

    String expected = set;
    testListSet(set, expected);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsSinglePageDC() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected single page (without token)");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListRecords", "oai_dc", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsMorePagesDC() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListRecords", "oai_dc", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsMorePagesDCFromUntil() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    // FIXME Values for vm1 only!
    String from = "2016-03-03";
    String until = "2018-05-04";
    int expectedPages = 5;

    int pageAmount = testList("ListRecords", "oai_dc", set, from, until);

    if (pageAmount != expectedPages) {
      assertTrue(false);
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsSinglePageDATACITE() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected single page (without token)");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListRecords", "oai_datacite", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsMorePagesDATASITE() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListRecords", "oai_datacite", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsMorePagesDATASITEFromUntil() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    // FIXME Values for vm1 only!
    String from = "2016-03-03";
    String until = "2018-05-04";
    int expectedPages = 5;

    int pageAmount = testList("ListRecords", "oai_datacite", set, from, until);

    if (pageAmount != expectedPages) {
      assertTrue(false);
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testGetRecordDC() throws IOException {

    System.out.println(TESTING + "#GETRECORD with expected single page (without token)");

    String identifier = "";
    String shouldContain = "";

    String url = host + "oai?verb=GetRecord&identifier=" + identifier + "&metadataPrefix=oai_dc";

    // FIXME Complete test!
  }

  /**
   * @throws IOException
   */
  @Test
  public void testGetRecordDATASITE() throws IOException {

    System.out.println(TESTING + "#GETRECORD with expected single page (without token)");

    String identifier = "";
    String shouldContain = "";

    String url =
        host + "oai?verb=GetRecord&identifier=" + identifier + "&metadataPrefix=oai_datacite";

    // FIXME Complete test!
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifierSinglePageDC() throws IOException {

    System.out.println(TESTING + "#LISTIDENTIFIERS with expected single page (without token)");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListIdentifiers", "oai_dc", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifiersMorePagesDC() throws IOException {

    System.out.println(TESTING + "#LISTIDENTIFIERS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListIdentifiers", "oai_dc", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifierSinglePageDATACITE() throws IOException {

    System.out.println(TESTING + "#LISTIDENTIFIERS with expected single page (without token)");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListIdentifiers", "oai_datacite", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifiersMorePagesDATASITE() throws IOException {

    System.out.println(TESTING + "#LISTIDENTIFIERS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    int pageAmount = testList("ListIdentifiers", "oai_datacite", set);
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * @param theUrl
   * @return
   * @throws IOException
   */
  private static TGHttpResponse getHttpResponse(String theUrl) throws IOException {

    try {
      Thread.sleep(TIME);
    } catch (InterruptedException e) {
      // Nothing interrupts here!
    }

    System.out.println("\trequest:  " + theUrl);

    URL url = new URL(theUrl);
    String host = url.getProtocol() + "://" + url.getHost();
    String query = url.getPath() + "?" + url.getQuery();
    TGHttpClient client = new TGHttpClient(host);

    // Get response.
    TGHttpResponse response = client.get(query);

    // Check HTTP status.
    int status = response.getStatusCode();
    String reason = response.getReasonPhrase();

    if (status != HttpStatus.SC_OK) {
      System.err.println("\tstatus:   " + status + " " + reason);
      System.err.println(ERROR);
      assertTrue(false);
    } else {
      System.out.println("\tstatus:   " + status + " " + reason);
    }

    return response;
  }

  /**
   * @param theResponse
   * @return Resumption token string, if existing, -1 if either no <resumptionToken> tag is
   *         existing, tag is existing and has no token value.
   * @throws IOException
   */
  private static String examineResumptionTokenTag(TGHttpResponse theResponse, String recordOrHeader,
      String oldtok) throws IOException {

    String res = IOUtils.readStringFromStream(theResponse.getInputstream());

    // System.out.println(res);

    // Test for OAIPMH errors.
    if (res.contains("<error code=\"badArgument\">")) {
      System.err.println(ERROR + " IN OAIPMH RESPONSE: " + res);
      assertTrue(false);
    }

    // Count response objects at first.
    int recordCount = 0;
    int i = res.indexOf("<" + recordOrHeader + ">", 0);
    while (i != -1) {
      recordCount++;
      i++;
      i = res.indexOf("<" + recordOrHeader + ">", i);
    }

    System.out.println("recordCount: " + recordCount);

    System.out.println("\t" + recordOrHeader + "s:  " + recordCount);

    // Check if token tag is existing.
    int tokStart = res.indexOf("<resumptionToken");
    int tokEnd = res.indexOf("</resumptionToken");

    if (tokStart == -1 && tokEnd == -1) {
      System.out.println("\ttoken:    no token");

      return "-1";
    }

    String restokTmp = res.substring(tokStart, tokEnd);
    // Get token tag.
    String toktag = restokTmp.substring(0, restokTmp.indexOf(">") + 1).trim();
    System.out.println("\ttokentag: " + toktag);

    // Get token.
    String restok = restokTmp.substring(restokTmp.indexOf(">") + 1).trim();
    System.out.println("\ttoken:    " + restok);

    // Check if old and new token are equal or not.
    boolean tokchanged = !oldtok.equals(restok);
    System.out.println("\ttokchngd: " + tokchanged);

    // Get completeListSize and cursor.
    String sizeStr = toktag.substring(toktag.indexOf("completeListSize=\"") + 18);
    int size = Integer.parseInt(sizeStr.substring(0, sizeStr.indexOf("\"")));
    String cursorStr = toktag.substring(toktag.indexOf("cursor=\"") + 8);
    int cursor = Integer.parseInt(cursorStr.substring(0, cursorStr.indexOf("\"")));
    System.out.println("\tsize:     " + size + " / " + cursor);

    // If token is provided, and we have less than 100 elements: mekkern!
    if (!restok.isEmpty()) {
      synchronized (OaiPmhDariahdeOnlineTests.class) {
        // Check <record> or <header> count, must be 100!
        if (recordCount != 100) {
          System.err.println(ERROR + ": " + recordOrHeader
              + " count mismatch, must be 100 if token is provided, but is " + recordCount);
        }
        if (size <= 100) {
          System.err.println(ERROR
              + ": completeListSize count mismatch, must be > 100 if token is provided, but is "
              + size);
        }
        if (recordCount != 100 || size <= 100) {
          assertTrue(false);
        }
      }
    }

    // If no token is provided, stop querying.
    else {
      // Check <record> count, must be completeListSize % 100.
      if (recordCount != size % 100) {
        System.err.println(ERROR + ": " + recordOrHeader + " count mismatch, should be "
            + size % 100 + ", but is " + recordCount);
        assertTrue(false);
      }

      // No resumption token available in response.
      return "-1";
    }

    System.out.println("\t------------");

    return restok;
  }

  /**
   * @param theMillis
   * @return
   */
  public static String getDurationInSecs(long theMillis) {

    int SECS_IN_MILLIS = 1000;
    int secs = SECS_IN_MILLIS;

    return theMillis / secs + " second" + ((theMillis / secs) != 1 ? "s" : "");
  }

  /**
   * @param theVerb
   * @param theMetadataPrefix
   * @param theSet
   * @return
   * @throws IOException
   */
  private static int testList(String theVerb, String theMetadataPrefix, String theSet)
      throws IOException {
    return testList(theVerb, theMetadataPrefix, theSet, null, null);
  }

  /**
   * @param theVerb
   * @param theMetadataPrefix
   * @param theSet
   * @param from
   * @param until
   * @return How many pages were delivered
   * @throws IOException
   */
  private static int testList(String theVerb, String theMetadataPrefix, String theSet, String from,
      String until)
      throws IOException {

    int result;

    long startTime = System.currentTimeMillis();

    String testOccurance = "header";
    if (theVerb.equals("ListRecords")) {
      testOccurance = "record";
    }

    String url = host + "oai?verb=" + theVerb + "&metadataPrefix=" + theMetadataPrefix;

    if (theSet != null && !theSet.isEmpty()) {
      url += "&set=" + theSet;
    }

    if (from != null && until != null) {
      url += "&from=" + from + "&until=" + until;
    }

    TGHttpResponse httpResponse = getHttpResponse(url);
    int status = httpResponse.getStatusCode();

    long timeRunning = System.currentTimeMillis() - startTime;
    System.out.println("\ttime:     " + getDurationInSecs(timeRunning));

    String restok = examineResumptionTokenTag(httpResponse, testOccurance, "");

    result = 1;
    while (status == HttpStatus.SC_OK && !restok.equals("-1")) {
      url = host + "/oai?verb=" + theVerb + "&resumptionToken=" + restok;
      httpResponse = getHttpResponse(url);
      timeRunning = System.currentTimeMillis() - startTime;
      System.out.println("\ttime:     " + getDurationInSecs(timeRunning));
      restok = examineResumptionTokenTag(httpResponse, testOccurance, restok);
      result++;
    }

    System.out.println(OK);

    return result;
  }

  /**
   * @param theSet
   * @throws IOException
   */
  private static void testListSet(String theSet, String theExpectedResponse) throws IOException {

    String verb = "ListSets";

    System.out.println(TESTING + "#LISTSETS");

    TGHttpResponse httpResponse = getHttpResponse(host + "oai?verb=" + verb);
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.contains(theExpectedResponse)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should contain '" + theExpectedResponse + "'");
      System.err.println(response);
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);

  }

  /**
   * @param theURL
   * @param shouldContain
   * @throws IOException
   */
  private static void testRootURL(String theURL, String shouldContain) throws IOException {

    System.out.println(TESTING + "#ROOTURL");

    TGHttpResponse httpResponse = getHttpResponse(host + theURL);
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.contains(shouldContain)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should contain '" + shouldContain + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);
  }

}
